//******************************************************************************
// Projeto - RELOGIO BINARIO V2 (NERD) - mostra a hora, minutos e segundos
// no formato binario, através de um painel (matriz) com 20 LEDs
//
// Essa versão difere da primeira em relação ao uso de um RTC para evitar
// precisar fazer o ajuste sempre que desligar ou quando temos uma falha na
// energia elétrica. Ela também foi desenvolvida com um outro compilador,
// o XC8 da Microchip.
//
// Desenvolvido por Márcio José Soares (V1 em 03/01/2006)
// Revisão V2 em 05/08/2022
//
// Arquivo - main.c (versão 2.0)
//
// Usa a frequencia de 60 Hz, retirada da rede elétrica para "marcar o tempo"
//
// Microcontrolador utilizado: PIC16F628-04/P (não testado com versão "A")
// Compilador: XC8 2.40
// IDE       : MPLABX
// Gravador  : PICKIT3
//
// Pinos utilizados:
// RA0 - controle da linha 1 na matriz de LEDs do mostrador
// RA1 - controle da linha 2 na matriz de LEDs do mostrador
// RA2 - controle da linha 3 na matriz de LEDs do mostrador
// RA3 - controle da linha 4 na matriz de LEDs do mostrador
// RA4 - modo ajuste (avança horario)
// RA5 - modo ajuste (muda de minutos para horas)
// RA6 - RESET do DS1302
// RA7 - não utilizado
//
// RB0 - entrada para clock 60 Hz (interrupção)
// RB1 - jumper para formato 24 horas
// RB2 - controle da coluna 1 na matriz de LEDs do mostrador
// RB3 - controle da coluna 2 na matriz de LEDs do mostrador
// RB4 - controle da coluna 3 na matriz de LEDs do mostrador e DT  DS1302
// RB5 - controle da coluna 4 na matriz de LEDs do mostrador e SCK DS1302
// RB6 - controle da coluna 5 na matriz de LEDs do mostrador
// RB7 - controle da coluna 6 na matriz de LEDs do mostrador
//
// Obs:  utilizado ULN2003 para controle das linhas da matriz de LEDs
//******************************************************************************

//******************************************************************************
// O Copyright deste programa está reservado para Márcio José Soares e seu
// conteúdo está protegido pela Lei de Direitos Autorais LEI N° 9.610,
// de 19 de Fevereiro de 1998. É estritamente proibida o uso comercial como
// também a publicação total ou parcial dos conteúdos aqui apresentados
// sem prévia autorização, por escrito, do autor.
//******************************************************************************

//******************************************************************************
// Arquivos incluídos neste módulo
//******************************************************************************
#include "relogio_binario.h"

//***********************************************************************
// Função para atualizar relógio
//
// Entradas - nenhuma
// Saídas   - nenhuma
//***********************************************************************
void atual_RTC(void){

    //LIN1 = 0;
    //LIN2 = 0;
    //LIN3 = 0;
    //LIN4 = 0;
    
	hora = horario.horas;           //variáveis hora, min e seg em DS1302.c
	min = horario.minutos;
	seg = horario.segundos;
    
	DS1302_set_time();				//atualiza RTC
    DS_RST = 0;                     //desliga o RTC
}

//******************************************************************************
// Função para desligar os LEDs (LINHAS e COLUNAS)
//
// Entradas - nenhuma
// Saídas 	- nenhuma
//******************************************************************************
void zera_lin_col(void){

    LIN1=0;                         //zera linhas
    LIN2=0;
    LIN3=0;
    LIN4=0;

    COL1=1;                         //zera colunas
    COL2=1;
    COL3=1;
    COL4=1;
    COL5=1;
    COL6=1;
    //CLRWDT();                     //zera WDT
}

//******************************************************************************
// Função para marcar qual ajuste será realizado
// Acende todos os LEDs da coluna 6, 4 ou 2
//
// Entradas - coluna
// Saídas 	- nenhuma
//******************************************************************************
void marca_ajuste(unsigned char col){

    LINHAS |= 0x0F;                 //todas linhas ligadas
    switch (col){
        case 2:
            COL2=0;
            break;
        case 4:
            COL4=0;
            break;
        case 6:
            COL6=0;
            break;
    }
    __delay_ms(1000);               //temporiza
    zera_lin_col();                 //zera os pinos de I/O do controle da matriz
    //CLRWDT();                     //zera WDT
}

//******************************************************************************
// Função para piscar todos os LEDs em sequencia
// Acende todos os LEDs da coluna 6 a coluna 1
//
// Entradas - nenhuma
// Saídas 	- nenhuma
//******************************************************************************
void pisca_ajuste(void){

    unsigned char i, j;

    j=0x80;
    LINHAS = 0x0F;                  //insere o valor máximo possível
    for(i=0;i<6;i++){
        COLUNAS = ~j;               //liga coluna
        __delay_ms(500);            //temporiza
        COLUNAS = 0xFF;             //desliga coluna
        j >>= 1;                    //desloca a direita
    }

    //CLRWDT();                     //zera WDT
}

//******************************************************************************
// Função para ajustar o horario
// Atualiza a matriz de LEDs a partir de zero hora
//
// Entradas - nenhuma
// Saídas 	- nenhuma
//******************************************************************************
void ajuste_horario(void){

   switch (est_ajt){                    //verifica estado inicial do ajuste
        case 0:                         //entrada no modo ajuste
            __delay_ms(300);
            INTCON &= ~0x40;            // PEIE=0 - desabilita int dos periféricos (timers, comparadores, usart, etc)
            INTCON &= ~0x80;            // GIE=0 - desabilita interrupções globais.
            zera_lin_col();             //zera os pinos de I/O do controle
                                        //da matriz

            if (prima)
                //pisca todos os LEDs para informar que aceitou
                //entrar no modo ajuste
                pisca_ajuste();
            else
                //se ajuste por falta de energia ou ainda por reset
                //não pisca para informar, e marca para próxima vez
                prima=1;

            horario.segundos=0;         //prepara para ajustar segundos
            horario.minutos=0;
            horario.horas=0;
            marca_ajuste(4);            //informa que vai ajustar minutos
            est_ajt=1;                  //muda máquina de estado
            break;

        case 1:
            if (AJT1 == 0){
                __delay_ms(300);
                ++horario.minutos;
                if (horario.minutos > 59)   //se igual a 60, zera
                    horario.minutos = 0;
            }
            if (AJT2 == 0){                 //S2 continua ajustes, horas
                __delay_ms(300);
                est_ajt=2;
                marca_ajuste(2);            //informa que vai ajustar horas
            }
            break;

        case 2:
            if (AJT1 == 0){
                __delay_ms(300);
                ++horario.horas;
                if (horario.horas > hora_final)     //se igual a 24 ou 12, zera
                    horario.horas = hora_inicial;
            }
            if (AJT2 == 0){                         //S2 encerra ajustes
                __delay_ms(300);
                est_ajt=0;
                conta=0;
                pisca_ajuste();         //pisca todos os LEDs para informar que aceitou
                                        //ajustes feitos
                atual_RTC();            //atualiza o RTC
                __delay_ms(200);
                INTCON |= 0x40;         // PEIE=1 - habilita int dos periféricos (timers, comparadores, usart, etc)
                INTCON |= 0x80;         // GIE=1 - habilita interrupções globais.
            }
            break;

    }
    //CLRWDT();	                   //zera WDT

}

//******************************************************************************
// Função para ligar/desligar matriz de LEDs
//
// Entradas - número da coluna a ser ligada
// Saídas 	- nenhuma
//******************************************************************************
void pisca(unsigned char nr){

    //CLRWDT();					     //zera WDT
    switch (nr){
      case 1:
            COL1=0;                    //liga coluna
            __delay_ms(tempo);         //temporiza
            COL1=1;                    //desliga coluna
            break;
      case 2:
            COL2=0;                 //liga coluna
            __delay_ms(tempo);         //temporiza
            COL2=1;                    //desliga coluna
            break;
      case 3:
            COL3=0;                    //liga coluna
            __delay_ms(tempo);         //temporiza
            COL3=1;                    //desliga coluna
            break;
      case 4:
            COL4=0;                    //liga coluna
            __delay_ms(tempo);         //temporiza
            COL4=1;                    //desliga coluna
            break;
      case 5:
            COL5=0;                    //liga coluna
            __delay_ms(tempo);         //temporiza
            COL5=1;                    //desliga coluna
            break;
      case 6:
            COL6=0;                    //liga coluna
            __delay_ms(tempo);         //temporiza
            COL6=1;                    //desliga coluna
            break;
   }
  //CLRWDT();					     //zera WDT
}

//******************************************************************************
// Função para atualizar a matriz
//
// Entradas - nenhuma
// Saídas 	- nenhuma
//******************************************************************************
void atualiza_matriz(void){

    //CLRWDT();                             //zera WDT
    LINHAS &= 0xF0;

    if (horario.segundos >= 10){            //é maior que dez
        LINHAS = horario.segundos % 10;     //atualiza unidade
        pisca(6);
        LINHAS = horario.segundos / 10;     //atualiza dezena
        pisca(5);
    }
    else{
        LINHAS = horario.segundos;        //atualiza unidade
        pisca(6);
        LINHAS = 0;
        pisca(5);
    }

    if (horario.minutos >= 10){
        LINHAS = horario.minutos % 10;
        pisca(4);
        LINHAS = horario.minutos / 10;
        pisca(3);
    }
    else{
        LINHAS = horario.minutos;
        pisca(4);
        LINHAS = 0;
        pisca(3);
    }

    if (horario.horas >= 10){
        LINHAS = horario.horas % 10;
        pisca(2);
        LINHAS = horario.horas / 10;
        pisca(1);
    }
    else{
        LINHAS = horario.horas;
        pisca(2);
        LINHAS = 0;
        pisca(1);
    }         
      
    //CLRWDT();	                 //zera WDT
}

//******************************************************************************
// Função inicializar variáveis globais
//
// Entradas - nenhuma
// Saídas 	- nenhuma
//******************************************************************************
void init_vars(void){
    
    horario.horas    = 0;               // zera horario
    horario.minutos  = 0;
    horario.segundos = 0;
    
    hora = 0;
    min = 0;
    seg = 0;
    
    conta=0;                            //zera contador de interrupções
    est_ajt=0;                          //inicia máquina de estado
    prima=0;                            //não configurou nada ainda
   
    if(F24H==0){                        //com jumper?
        hora_final = HF12;              //formato 12 horas
        hora_inicial= HI12;
        tipohora = 0x80;
    }
    else{                               //sem jumper?
        hora_final = HF24;              //formato 24 horas
        hora_inicial = HI24;
        tipohora = 0x00;
    }
    
}

//******************************************************************************
// Função principal
//
// Entradas - nenhuma
// Saídas 	- nenhuma
//******************************************************************************
void main(void) {
    
    //**************************************************************************
    // Prepara microcontrolador
    CMCON = 0x07;                    	//desliga comparadores
    INTE  = 0;                    		//desabilita a interrupção no pino RB0
    TMR1IE= 0;
    
    //**************************************************************************
    // Prepara pinos de I/O
    TRISA = 0b00110000;                 //configura pinos do port A como saída,
                                        //exceto RA4 e RA5
    
    TRISB = 0b00000011;                 //configura pinos do port B como saída,
                                        //exceto RB0, RB1    
    //**************************************************************************
    // Prepara int ext
    init_int_ext();
    
    //**************************************************************************
    // Seta vars globais
    init_vars();  
    __delay_ms(500);                     // aguarda antes de inicializar
    
    //**************************************************************************
    // Prepara DS1302    
    DS1302_init();                      //inicia o DS1302
    __delay_ms(100);
   
    DS1302_get_time();                  //colhe horário no RTC
	horario.horas = hora;               //atualiza var com hora inserida no RTC
	horario.minutos = min;
	horario.segundos = seg;
   
    DS_RST = 0;                         //desliga RTC!

    //CLRWDT();                         //zera WDT
    //**************************************************************************
    // Flags de controle das interrupções
    INTCON |= 0x40;                     // PEIE=1 - habilita int dos periféricos (timers, comparadores, usart, etc)
    INTCON |= 0x80;                     // GIE=1 - habilita interrupções globais.
    //CLRWDT();
    
    zera_lin_col();                     //limpa pinos de I/O de controle da
    
    while(TRUE){                        //faz eternamente

        //entra no modo ajuste
        if (AJT1 == 0 || AJT2 == 0)
            ajuste_horario();           //ajusta horário

        atualiza_matriz();          //atualiza matriz de LEDs
        CLRWDT();	               //zera WDT
    }
    
    return;
}